import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { SideNav } from "./sidenav.po";
import { browser } from "protractor";
import { PreferencesPage } from "./preferences.po";

/**
 * Load a session containing 5 calculators, having multiple linked parameters
 * from one to another
 * @TODO les valeurs des Select sont comparées au français, pas très générique :/
 */
describe("ngHyd − load session with multiple linked parameters − ", () => {
    let startPage: AppPage;
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let prefPage: PreferencesPage;
    let navbar: Navbar;
    let sidenav: SideNav;

    function init() {
        startPage = new AppPage();
        calcPage = new CalculatorPage();
        prefPage = new PreferencesPage();
        navbar = new Navbar();
        sidenav = new SideNav();
        listPage = new ListPage();
    }
    beforeAll(init);

    beforeEach(async () => {
        // force language to prevent issues due to default browser language
        await prefPage.navigateTo();
        await prefPage.changeLanguage(1); // fr
        await browser.sleep(200);
        await navbar.clickNewCalculatorButton();
    });

    it("when loading session-liens-spaghetti.json, all links should point to the right target", async () => {
        await navbar.clickMenuButton();
        await browser.sleep(200);

        await sidenav.clickLoadSessionButton();
        await browser.sleep(200);

        await sidenav.loadSessionFile("./session/session-liens-spaghetti.json");
        await browser.sleep(500);

        expect(await navbar.getAllCalculatorTabs().count()).toBe(5);

        // 1. check Section paramétrée
        await navbar.clickCalculatorTab(0);
        await browser.sleep(500);

        // check target params values
        const sp_lb = calcPage.getSelectById("linked_LargeurBerge");
        const sp_lbv = await calcPage.getSelectValueText(sp_lb);
        expect(sp_lbv).toContain("Largeur du déversoir (Ouvrages, ouvrage 3)");

        // 2. check Passe à macro-rugosités
        await navbar.clickCalculatorTab(1);
        await browser.sleep(500);

        // check target params values
        const mr_zf1 = calcPage.getSelectById("linked_ZF1");
        const mr_zf1v = await calcPage.getSelectValueText(mr_zf1);
        expect(mr_zf1v).toContain("Cote de l'eau amont (Ouvrages)");

        const mr_q = calcPage.getSelectById("linked_Q");
        const mr_qv = await calcPage.getSelectValueText(mr_q);
        expect(mr_qv).toContain("Débit (Sec. param., section)");

        // 3. check Lois d'ouvrages
        await navbar.clickCalculatorTab(2);
        await browser.sleep(500);

        // check target params values
        const lo_z2 = calcPage.getSelectById("linked_Z2");
        const lo_z2v = await calcPage.getSelectValueText(lo_z2);
        expect(lo_z2v).toContain("Cote de fond aval (Macro-rugo.)");

        const lo_l = calcPage.getSelectById("1_linked_L");
        const lo_lv = await calcPage.getSelectValueText(lo_l);
        expect(lo_lv).toContain("Largeur au miroir (Sec. param.)");

        const lo_w = calcPage.getSelectById("2_linked_W");
        const lo_wv = await calcPage.getSelectValueText(lo_w);
        expect(lo_wv).toContain("Ouverture de vanne (Ouvrages, ouvrage 2)");

        // 5. check Déver. dénoyés
        await navbar.clickCalculatorTab(4);
        await browser.sleep(500);

        // check target params values
        const lo_br = calcPage.getSelectById("linked_BR");
        const lo_brv = await calcPage.getSelectValueText(lo_br);
        expect(lo_brv).toContain("Largeur au miroir (Sec. param.)");
    });

});
