import { FormulaireParallelStructure } from "./form-parallel-structures";
import { SelectFieldNub } from "../elements/select-field-nub";

import { IObservable, Nub, PbCloison, PbBassin } from "jalhyd";

export class FormulairePbCloison extends FormulaireParallelStructure {

    /** id of select configuring upstream basin Nub */
    private _upstreamBasinSelectId: string;

    /** id of select configuring downstream basin Nub */
    private _downstreamBasinSelectId: string;

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._upstreamBasinSelectId = this.getOption(json, "upstreamBasinSelectId");
        this._downstreamBasinSelectId = this.getOption(json, "downstreamBasinSelectId");
    }

    protected completeParse(json: {}) {
        super.completeParse(json);
        if (this._upstreamBasinSelectId) {
            const sel = this.getFormulaireNodeById(this._upstreamBasinSelectId);
            if (sel) {
                sel.addObserver(this);
            }
        }
        if (this._downstreamBasinSelectId) {
            const sel = this.getFormulaireNodeById(this._downstreamBasinSelectId);
            if (sel) {
                sel.addObserver(this);
            }
        }
    }

    // interface Observer

    public update(sender: IObservable, data: any) {
        super.update(sender, data);

        if (sender instanceof SelectFieldNub) {
            const nub = this._currentNub as PbCloison;
            const pb = nub.parent;
            // empty "" data.value.value should return undefined, which is good for amont/aval
            const newBasin = pb.findChild(data.value.value) as PbBassin;
            if (sender.id === this._upstreamBasinSelectId) {
                // remove and recreate wall (easier for pointers consistency)
                const oldDownstreamBasin = nub.bassinAval;
                pb.deleteChild(pb.findChildPosition(nub.uid));
                const newWall = new PbCloison(newBasin, oldDownstreamBasin);
                // copy structures
                for (const s of nub.structures) {
                    newWall.addChild(s);
                }
                pb.addChild(newWall);
                this.currentNub = newWall;
            } else if (sender.id === this._downstreamBasinSelectId) {
                // remove and recreate wall (easier for pointers consistency)
                const oldUpstreamBasin = nub.bassinAmont;
                pb.deleteChild(pb.findChildPosition(nub.uid));
                const newWall = new PbCloison(oldUpstreamBasin, newBasin);
                // copy structures
                for (const s of nub.structures) {
                    newWall.addChild(s);
                }
                pb.addChild(newWall);
                this.currentNub = newWall;
            }
            this.notifyObservers({
                action: "updateBasin",
                value: this._currentNub.uid // node to select on the schema
            }, this);
        }
    }
}
