import { Nub, capitalize } from "jalhyd";

import { NgParameter } from "../formulaire/elements/ngparam";
import { ServiceFactory } from "../services/service-factory";

import { sprintf } from "sprintf-js";

export abstract class CalculatorResults {

    /** help links by result symbol, read from calculator config */
    public helpLinks: { [key: string]: string };

    /**
     * @param p parameter to generate label for
     * @param displaySymbol if true, will prefix label with parameter symbol (ex: "ZDV")
     * @param referenceNub if given, will detect if parameter belongs to a child of this reference Nub, and
     *                     if so will add a number prefix (ex: "0_") before the label
     */
    public static paramLabel(p: NgParameter, displaySymbol: boolean, referenceNub?: Nub): string {
        let res = "";
        // detect child parameter
        let isChildParam = false;
        if (referenceNub) {
            const children = referenceNub.getChildren();
            const parameterNub = p.paramDefinition.parentNub;
            if (children.includes(parameterNub)) {
                const cn = capitalize(ServiceFactory.i18nService.childName(parameterNub.parent));
                isChildParam = true;
                const pos = parameterNub.findPositionInParent() + 1;
                res = sprintf(ServiceFactory.i18nService.localizeText("INFO_STUFF_N"), cn) + pos + " : ";
            }
        }
        if (displaySymbol && ! isChildParam) {
            res += p.symbol;
        }
        if (p.label !== undefined && p.label !== "") {
            if (displaySymbol && ! isChildParam) {
                res += " : ";
            }
            res += p.label;
        }
        if (p.unit !== undefined && p.unit !== "") {
            res += " (" + p.unit + ")";
        }
        return res;
    }

    /**
     * remet tous les résultats à zero
     */
    protected abstract reset();

    /**
     * indique si il existe des résultats à afficher
     */
    public abstract get hasResults(): boolean;

    public getHelpLink(symbol: string): string {
        // add help link if any
        if (this.helpLinks !== undefined && this.helpLinks[symbol] !== undefined) {
            const helpURL = "assets/docs/" + ServiceFactory.applicationSetupService.language
                + "/calculators/" + this.helpLinks[symbol];
            // pseudo-<mat-icon> dirty trick because <mat-icon> renderer cannot be
            // triggered when code is set through innerHTML
            return `<a href="${helpURL}" target="_blank"><span class="material-icons mat-accent">help</span></a>`;
        } else {
            return "";
        }
    }
}
